<?php

namespace DMG\JSON_Config;
/**
 * Class Editor
 *
 * Register a new JSON config editor
 *
 * @package DMG\JSON_Config
 */
class Editor {

	/**
	 * Config save nonce
	 *
	 * @var string
	 */
	const NONCE = 'json_config_save';

	private $key;
	private $slug;
	private $name;
	private $cap;

	/**
	 * Constructor
	 *
	 * @param string $key   The config key
	 * @param string $label The config label
	 */
	public function __construct( string $key, string $label ) {
		$this->key = $key;
		$this->slug = $key . 'config';
		$this->name = $label;
		$this->cap = 'manage_' . $this->slug;

		$this->add_actions();
	}

	/**
	 * Bind actions
	 */
	private function add_actions() {
		add_action( 'init', [$this, 'register_post_type']);
		add_action( 'rest_api_init', [$this, 'register_route'] );
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_enqueue_scripts' ] );
		add_action( 'admin_menu', [ $this, 'admin_menu' ] );
		add_action( 'admin_post_' . $this->slug . '_config-save', [ $this, 'save' ] );
		add_action( 'wp_ajax_' . $this->slug . '_config-save', [ $this, 'save' ] );
		add_action( 'admin_init', [ $this, 'add_capability'] );
	}

	/**
	 *  Register the config post types
	 */
	public function register_post_type() {
		register_post_type(
			$this->slug, array(
				'labels'           => array(
					'name'          => $this->name,
					'singular_name' => $this->name,
				),
				'public'           => false,
				'hierarchical'     => false,
				'rewrite'          => false,
				'query_var'        => false,
				'delete_with_user' => false,
				'supports'         => array( 'revisions' ),
				'capabilities'     => array(
					'create_posts'           => $this->cap,
					'delete_others_posts'    => $this->cap,
					'delete_post'            => $this->cap,
					'delete_posts'           => $this->cap,
					'delete_private_posts'   => $this->cap,
					'delete_published_posts' => $this->cap,
					'edit_others_posts'      => $this->cap,
					'edit_post'              => $this->cap,
					'edit_posts'             => $this->cap,
					'edit_private_posts'     => $this->cap,
					'edit_published_posts'   => $this->cap,
					'publish_posts'          => $this->cap,
					'read'                   => 'read',
					'read_post'              => $this->cap,
					'read_private_posts'     => $this->cap,
				),
			)
		);
	}

	/**
	 * Register endpoint to return JSON configs
	 *
	 * e.g. /wp-json/json-config/app
	 *      /wp-json/json-config/ads
	 */
	public function register_route() {
		register_rest_route(
			'json-config',
			'/'. $this->key .'/',
			array(
				'methods'             => 'GET',
				'callback'            => function() {
					return json_decode( get($this->key) );
				},
				'permission_callback' => '__return_true',
			)
		);
	}

	/**
	 * Add admin menu page.
	 *
	 * @return void
	 */
	function admin_menu() {
		/**
		 * Changing the capability in add_options_page causes unexpected behaviour such as  other options pages missing
		 * from the settings menu and inability to access the added options page even with the stated capability.
		 *
		 * Thus to circumnavigate those issues we'll only add the options page if the user has the needed cap and when
		 * we do register it, it is with the a simple capability.
		 */
		 if(current_user_can($this->cap)){
			add_options_page( $this->name, $this->name, 'read', $this->slug . '-settings', [ $this, 'settings_screen' ] );
		}
	}

	/**
	 * Enqueue the admin assets
	 *
	 * @param $hook
	 */
	public function admin_enqueue_scripts( $hook ) {

		if ( 'settings_page_'. $this->slug .'-settings' !== $hook ) {
			return;
		}

		wp_enqueue_script(
			'jsonconfig',
			plugin_dir_url( __FILE__ ) . 'js/admin.js',
			[ 'jquery', 'jsoneditor', 'wp-backbone' ],
			false,
			true
		);

		wp_enqueue_script(
			'jsoneditor',
			plugin_dir_url( __FILE__ ) . 'jsoneditor/jsoneditor.min.js',
			[],
			false,
			true
		);

		wp_enqueue_style(
			'jsoneditor-style',
			plugin_dir_url( __FILE__ ) . 'jsoneditor/jsoneditor.min.css',
			[],
			false,
			'screen'
		);

	}

	/**
	 * Output the settings screen.
	 *
	 * @return void
	 */
	public function settings_screen() {
		$post_id = get_option( $this->slug . '_post' );
		$post    = false;
		$content = false;

		if ( $post_id ) {
			$post = get_post( $post_id );
		}

		if ( is_a( $post, 'WP_Post' ) ) {
			$content = $post->post_content;
		}
		?>
		<div class="wrap">

			<h2>Manage JSON Config</h2>

			<form method="post" action="<?php echo esc_url( admin_url( 'admin-post.php' ) ); ?>" id="json_config-settings-form" class="json_config-settings-form">
				<input type="hidden" name="post_id" value="<?php echo ( is_a( $post, 'WP_Post' ) ? esc_attr( $post->ID ) : '' ); ?>" />
				<input type="hidden" name="post_type" value="<?php echo esc_attr( $this->slug); ?>" />
				<input type="hidden" name="action" value="<?php echo esc_attr( $this->slug); ?>_config-save" />
				<input type="hidden" id="json_config_content" name="json_config" value="" />
				<?php wp_nonce_field( self::NONCE ); ?>

				<div id="jsoneditor" style="width: 100%; height: 100%; min-height: 400px;" data-content="<?php echo esc_attr( $content ); ?>"></div>

				<div id="json_config-notification-area"></div>

				<p class="submit">
					<input type="submit" name="submit" id="submit" class="button button-primary" value="<?php echo esc_attr( 'Save Changes' ); ?>">
				</p>

			</form>
		</div>

		<?php
	}

	/**
	 * Take an error array and turn it into a message.
	 *
	 * @param  array $error {
	 *     Array of error message components.
	 *
	 *     @type string $type    Type of error.
	 * }
	 *
	 * @return string       Formatted error message.
	 */
	public function format_error( $error ) {
		$messages = $this->get_error_messages();

		if ( ! isset( $messages[ $error['type'] ] ) ) {
			return 'Unknown error';
		}

		$message = $messages[ $error['type'] ];

		return $message;
	}

	/**
	 * Get all non-generic error messages
	 *
	 * @return array Associative array of error messages.
	 */
	public function get_error_messages() {
		$messages = array(
			'invalid_json' => 'JSON cannot be parsed',
		);

		return $messages;
	}

	/**
	 * Process and save the json config data.
	 *
	 * Handles both AJAX and POST saves via `admin-ajax.php` and `admin-post.php` respectively.
	 * AJAX calls output JSON; POST calls redirect back to the Ads Config edit screen.
	 *
	 * @return void
	 */
	public function save() {
		current_user_can( $this->cap ) || die;
		check_admin_referer( self::NONCE );
		$_post      = stripslashes_deep( $_POST );
		$doing_ajax = defined( 'DOING_AJAX' ) && DOING_AJAX;

		$post_id = $_post['post_id'];
		$post_type = $_post['post_type'];
		$ays     = isset( $_post['json_config_ays'] ) ? $_post['json_config_ays'] : null;

		$sanitized = wp_strip_all_tags( $_post['json_config'] );
		$errors    = [];
		$response  = [];

		// Validate the config
		$json_config_json = json_decode( $sanitized );

		if ( null === $json_config_json && JSON_ERROR_NONE !== json_last_error() ) {
			$errors[] = [
				'type' => 'invalid_json',
			];
		}

		$post_data = [
			'ID'           => $post_id,
			'post_title'   => $post_type,
			'post_content' => $sanitized,
			'post_type'    => $post_type,
			'post_status'  => 'publish',
		];

		if ( ! $doing_ajax || empty( $errors ) || 'y' === $ays ) {
			$post_id = wp_insert_post( $post_data );

			if ( !empty($post_id) ) {
				update_option( $this->slug . '_post', $post_id, false );
				$response['saved'] = true;
			}
		}

		if ( $doing_ajax ) {
			$response['sanitized'] = $sanitized;

			if ( ! empty( $errors ) ) {
				$response['errors'] = $errors;
			}

			echo wp_json_encode( $response );
			exit;
		}

		wp_safe_redirect( esc_url_raw( $_post['_wp_http_referer'] ) . '&updated=true' );
		exit;
	}

	/**
	 * Add capability to admins on dashboard load
	 */
	public function add_capability() {
		$role_object = get_role( 'administrator' );
		$role_object->add_cap( $this->cap );
	}
}
